-- Nécessite la bibliothèque d'importation
require "import"
-- Importe les packages d'interface utilisateur Android
import "android.widget.*"
-- Importe le package AndroLua
import "com.androlua.*"
-- Importe la classe View
import "android.view.View"
-- Importe la classe Environment pour l'accès au stockage externe
import "android.os.Environment"
-- Importe la classe URLEncoder pour l'encodage des URL
import "java.net.URLEncoder"
-- Importe la classe File pour la manipulation des fichiers
import "java.io.File"
-- Importe la classe FileOutputStream pour l'écriture de fichiers
import "java.io.FileOutputStream"
-- Importe la classe BitmapFactory pour le décodage des bitmaps
import "android.graphics.BitmapFactory"

-- Fonction pour obtenir un nom de fichier unique (image IA)
function getUniqueFilename(name)
  local t = os.date("*t") -- Obtient l'heure actuelle sous forme de table
  -- Formate l'horodatage pour l'unicité
  local ts = string.format("%04d%02d%02d_%02d%02d%02d", t.year, t.month, t.day, t.hour, t.min, t.sec)
  -- Construit le chemin complet du fichier dans le stockage externe
  return Environment.getExternalStorageDirectory().toString() .. "/" .. name .. "_" .. ts .. ".png"
end

-- Fonction pour télécharger l'image depuis l'API IA
function downloadImage(text, callback)
  -- Encode le texte pour l'utilisation dans l'URL
  local encoded = URLEncoder.encode(text, "UTF-8")
  -- Construit l'URL de l'API de génération d'art IA
  local url = "https://sheikhhridoy.nagad.my.id/api/ai-art-generator.php?prompt=" .. encoded
  -- Obtient un chemin unique pour enregistrer l'image
  local path = getUniqueFilename("image_ia") -- Nom de fichier de base "image_ia"
  -- Exécute une requête HTTP GET pour télécharger l'image
  Http.get(url, nil, function(code, content)
    if code == 200 and content then
      -- Ouvre un fichier en mode écriture binaire
      local f = io.open(path, "wb")
      if f then
        f:write(content) -- Écrit le contenu de l'image dans le fichier
        f:close() -- Ferme le fichier
        callback(true, path) -- Appelle le rappel avec succès et le chemin du fichier
      else
        callback(false, "Écriture échouée.") -- Appelle le rappel avec échec d'écriture
      end
    else
      callback(false, "Erreur : " .. code) -- Appelle le rappel avec le code d'erreur
    end
  end)
end

-- Définit le layout de l'interface utilisateur (UI)
layout = {
  LinearLayout, -- Conteneur de layout vertical
  orientation = "vertical", -- Orientation verticale
  layout_width = "fill", -- Largeur remplissant le parent
  layout_height = "fill", -- Hauteur remplissant le parent
  padding = "16dp", -- Rembourrage intérieur de 16dp
  {
    EditText, -- Champ de saisie de texte
    id = "inputText", -- ID unique pour le champ de texte
    hint = "Entrez une description", -- Indication affichée à l'utilisateur
    layout_width = "fill", -- Largeur remplissant le parent
    layout_height = "wrap_content", -- Hauteur s'adaptant au contenu
  },
  {
    Button, -- Bouton pour générer l'image
    id = "generateButton", -- ID unique pour le bouton
    text = "Générer l'image", -- Texte du bouton
    layout_width = "fill", -- Largeur remplissant le parent
    layout_height = "wrap_content", -- Hauteur s'adaptant au contenu
  },
  {
    TextView, -- Champ de texte pour afficher les résultats ou les messages
    id = "resultText", -- ID unique pour le champ de texte
    layout_width = "fill", -- Largeur remplissant le parent
    layout_height = "wrap_content", -- Hauteur s'adaptant au contenu
  },
  {
    ImageView, -- Pour afficher l'image générée
    id = "generatedImage", -- ID unique pour le champ d'image
    layout_width = "fill", -- Largeur remplissant le parent
    layout_height = "300dp", -- Hauteur fixe de 300dp
    visibility = View.GONE, -- Cachée au début
  },
  {
    Button, -- Bouton de sortie
    id = "exitButton", -- ID unique pour le bouton de sortie
    text = "Quitter", -- Texte du bouton
    layout_width = "wrap_content", -- Largeur s'adaptant au contenu
    layout_height = "wrap_content", -- Hauteur s'adaptant au contenu
  },
  {
    TextView, -- Texte affichant le lien de la chaîne Telegram
    text = "https://t.me/+dJrRrA6wxNlmZmU0", -- Lien de la chaîne
    layout_width = "fill", -- Largeur remplissant le parent
    layout_height = "wrap_content", -- Hauteur s'adaptant au contenu
    gravity = "center", -- Alignement du texte au centre
    padding = "8dp", -- Rembourrage intérieur de 8dp
  }
}

-- Crée une boîte de dialogue Lua
dlg = LuaDialog(this)
dlg.setTitle("Générateur d'images IA SAYAHFETHI") -- Définit le titre de la boîte de dialogue
dlg.setView(loadlayout(layout)) -- Charge le layout dans la boîte de dialogue

-- Fonction pour mettre à jour le texte du résultat
function updateResult(t)
  resultText.text = t or ""
end

-- Gestionnaire de clic pour le bouton "Générer l'image"
generateButton.onClick = function()
  local prompt = inputText.text -- Obtient le texte saisi par l'utilisateur
  if prompt and #prompt > 0 then
    updateResult("Génération en cours...") -- Affiche le message "Génération en cours"
    generatedImage.visibility = View.GONE -- Cache l'image précédente
    downloadImage(prompt, function(ok, result) -- Appelle la fonction de téléchargement d'image
      if ok then
        local bmp = BitmapFactory.decodeFile(result) -- Décode l'image à partir du fichier
        if bmp then
          generatedImage.setImageBitmap(bmp) -- Définit l'image générée dans l'ImageView
          generatedImage.visibility = View.VISIBLE -- Rend l'image visible
          updateResult("Enregistré à l'emplacement :\n" .. result) -- Affiche le message de succès et le chemin d'enregistrement
        else
          updateResult("Échec du décodage.") -- Affiche le message d'échec de décodage
        end
      else
        updateResult(result) -- Affiche le message d'erreur
      end
    end)
  else
    updateResult("Veuillez entrer une description.") -- Demande à l'utilisateur de saisir une description
  end
end

-- Gestionnaire de clic pour le bouton "Quitter"
exitButton.onClick = function()
  dlg.dismiss() -- Ferme la boîte de dialogue
end

-- Affiche la boîte de dialogue
dlg.show()